#!/usr/bin/env python

"""
Common handler functionality for different entities.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.content import to_part

class SupportFreebusy:

    "Support for free/busy publishing and sharing."

    def make_freebusy_to_publish(self, from_organiser=True):

        """
        Make a freebusy object for publication for a user, providing either an
        organiser's details if 'from_organiser' is set to a true value, or an
        attendee's details otherwise.
        """

        calendar = self.make_freebusy(from_organiser, publish=True)

        # Return a published object.

        if calendar:
            return [(True, to_part("PUBLISH", calendar))]

    def make_freebusy(self, from_organiser=True, publish=False):

        """
        Make a freebusy object, providing either an organiser's details if
        'from_organiser' is set to a true value, or an attendee's details
        otherwise.
        """

        oa = self.require_organiser_and_attendees()
        if not oa:
            return None

        (organiser, organiser_attr), attendees = oa

        # Get the details for each attendee.

        calendar = []
        cwrite = calendar.append

        for attendee, attendee_attr in attendees.items():

            # Construct an appropriate fragment.

            freebusy = self.store.get_freebusy(from_organiser and organiser or attendee)

            record = []
            rwrite = record.append

            # For replies, the organiser is preserved.

            if not publish or from_organiser:
                rwrite(("ORGANIZER", organiser_attr, organiser))

            # For published objects, the organiser is actually the user whose
            # information is being provided.

            else:
                rwrite(("ORGANIZER", attendee_attr, attendee))

            # For replies, the attendee is preserved.
            # (Published objects do not employ the attendee property.)

            if not publish:
                rwrite(("ATTENDEE", attendee_attr, attendee))

            rwrite(("UID", {}, self.uid))

            if freebusy:
                for start, end, uid, transp in freebusy:
                    if transp == "OPAQUE":
                        rwrite(("FREEBUSY", {"FBTYPE" : "BUSY"}, "/".join([start, end])))

            cwrite(("VFREEBUSY", {}, record))

        # Return the object.

        return calendar

class CommonFreebusy(SupportFreebusy):

    "Common free/busy mix-in."

    def request(self):

        """
        Respond to a request by preparing a reply containing free/busy
        information for each indicated attendee.
        """

        calendar = self.make_freebusy(from_organiser=False)

        # Return the reply.

        return [(True, to_part("REPLY", calendar))]

# vim: tabstop=4 expandtab shiftwidth=4
